function fm = glps(X,Y,loglambda0,nvargroup)
%Fit a partitioned LASSO-Patternsearch model. X is divided into small groups,
%each with nvargroup variables. LPS is fitted on each between-group step
%and within-group step. A final LPS model with 2 tuning parameters is
%fitted on the surviving variables. 
%The default of nvargroup is 200.
%X is predictor matrix and must be binary with values 0 and 1
%Y is binary response column vector taking values 0 and 1
global fBasis;
global Y_global;

bgacv = 1; output = 0;
Y_global = Y;
[sampsize dim] = size(X);

% did the user supply a regularization parameter for the group stage?
% If not, set a default.
if nargin < 3
    loglambda0 = -1.1;
end

% did the user supply a group size? If not, set a default.
if nargin < 4
    nvargroup = 200;
end

fprintf(1,'\n\n');
fprintf(1,'Problem dimensions: %d x %d;  vars per group: %d, log(reg param): %6.2f\n', ...
   size(X),nvargroup, loglambda0);

ngroup1 = floor(dim/nvargroup);
inx.main = [];
inx.inter = [];

t1 = clock;
kk = 0;
%--------------------------------------------------------------------------
% On all squares
%--------------------------------------------------------------------------
for i = 1:ngroup1-1
    for j = i+1:ngroup1
        [fBasis inxfullcurr] = gBasisg(X,nvargroup,i,j,1);
        kk = kk + 1;
        npar = size(fBasis,2);
        fprintf(1,'Square subproblem %3d, number of parameters = %5d\n',...
               kk,npar);     
        beta = betaopt(loglambda0, bgacv, output);
        inxcurr = getinx(beta,inxfullcurr);
        inx.main = [inx.main; inxcurr.main];
        inx.inter = [inx.inter; inxcurr.inter];
    end
end

%--------------------------------------------------------------------------
% On major triangles
%--------------------------------------------------------------------------
for i = (2:2:ngroup1) - 1
    [fBasis inxfullcurr] = gBasisg(X,nvargroup,i,i+1,2);
    kk = kk + 1;
    npar = size(fBasis,2);
    fprintf(1,'Triangle subproblem %3d, number of parameters = %5d\n',kk,npar);     
    beta = betaopt(loglambda0, bgacv, output);
    inxcurr = getinx(beta,inxfullcurr);
    inx.main = [inx.main; inxcurr.main];
    inx.inter = [inx.inter; inxcurr.inter];
end

%--------------------------------------------------------------------------
% On remaining rectangles
%--------------------------------------------------------------------------
for i = 1:ngroup1
    j = ngroup1 + 1;
    [fBasis inxfullcurr] = gBasisg(X,nvargroup,i,j,3);
    kk = kk + 1;
    npar = size(fBasis,2);
    fprintf(1,'Leftover rectangle subproblem %3d, number of parameters = %5d\n',...
       kk,npar);     
    beta = betaopt(loglambda0, bgacv, output);
    inxcurr = getinx(beta,inxfullcurr);
    inx.main = [inx.main; inxcurr.main];
    inx.inter = [inx.inter; inxcurr.inter];
end

%--------------------------------------------------------------------------
% On remaining triangles
%--------------------------------------------------------------------------
if mod(ngroup1,2) == 1      % big&small
    i = ngroup1;
    j = ngroup1 + 1;
    [fBasis inxfullcurr] = gBasisg(X,nvargroup,i,j,4);
else                        % small
    i = ngroup1 + 1;
    j = ngroup1 + 2;
    [fBasis inxfullcurr] = gBasisg(X,nvargroup,i,j,5);
end
kk = kk + 1;
npar = size(fBasis,2);
fprintf(1,'Leftover triangular subproblem %3d, number of parameters = %5d\n',...
      kk,npar);     
beta = betaopt(loglambda0, bgacv, output);
inxcurr = getinx(beta,inxfullcurr);
inx.main = [inx.main; inxcurr.main];
inx.inter = [inx.inter; inxcurr.inter];
 
%--------------------------------------------------------------------------
% Solve a full model over the variables selected in the group stage
%--------------------------------------------------------------------------
if size(inx.inter,1) > 1
    inxg = unique([unique(inx.main); unique(inx.inter)]);
else
    inxg = unique([unique(inx.main)' unique(inx.inter)])';
                            % indices of variables selected by group step
end

fprintf(1,'Fitting a full model on variables selected by subproblems\n');
fprintf(1,'\n %d main vars, %d combined effects, group loglambda0=%6.2f\n', ...
     length(inx.main), length(inx.inter), loglambda0);
if (length(inx.main)+length(inx.inter)==0)
  fprintf(1,[' No variables selected at group stage! Need smaller ' ...
             'loglambda0\n']);
  fm.inxg=[];
  fm.model=[];
  fm.pattern=[];
  info.haspattern=[];
else
  % call top-stage problem to determine final selection
  fmlps = lpsmt(X(:,inxg),Y);
  inx.main = unique(inx.main);
  fm.inxg = inx;
  % return fm
  model.coeff = fmlps.model.coeff;
  model.se = fmlps.model.se;
  model.coeffcorr = fmlps.model.coeffcorr;
  model.p = fmlps.model.p;
  model.prob = fmlps.model.prob;
  fm.model = model;
  pattern = fmlps.pattern;
  pattern(pattern>0) = inxg(pattern(pattern>0));
  fm.pattern = pattern;
  info.haspattern = fmlps.info.haspattern;
end

t2 = clock;
info.size = size(X,1);
info.order = 2;
info.runtime = etime(t2,t1);
fm.info = info;
return;

%--------------------------------------------------------------------------
% Get the index of main effects and interactions, after calculating beta
%--------------------------------------------------------------------------
function inx = getinx(beta,inxfullcurr)
global fBasis Y_global
nmainfullcurr = length(inxfullcurr.main);
inx.main = [];
inx.inter = [];
inxlasso = find(abs(beta)>1e-6)';
nsigblasso = length(inxlasso);
if nsigblasso > 1
    inxlasso = inxlasso(1:nsigblasso-1);
    XX = fBasis(:,inxlasso);
    [ncoe nstat inxlogist] = bgacvp(XX,Y_global);
    inxlps = inxlasso(inxlogist);
else
    inxlps = [];
end
ninxlps = length(inxlps);
if ninxlps > 0
    inx.main = inxfullcurr.main(intersect(1:nmainfullcurr,inxlps));
    intertmp = setdiff(inxlps,1:nmainfullcurr) - nmainfullcurr;
    inx.inter = inxfullcurr.inter(intertmp,:);
end
return
    
