function fm = lpsmt(X,Y,order_inter,loglambda0,bgacv,output)
%Peform Lasso-Patternsearch algorithm with seperate tuning parameters for
%main effects and second order (and third order) interactions.
%When third order interactions are included, it shares the same smoothing 
%paramter with the second order interactions. 
%
%fm = lpsmt(X,Y) fits a LASSO-Patternsearch model using the predictor matrix
%X, response Y. All second order interactions of X are included. 
%X is a binary matrix, coded as (0,1) or (-1,1). Y is also binary and coded
%as (0,1).
%
%order_inter is limited to 2 or 3. Default is 2. 
%
%loglambda0 (2x1) is the initial point of log10 of the smoothing parameter 
%when bgacv = 1; and is the user selected log10 of the smoothing parameter 
%when bgacv = 0.
%
%bgacv = 1 selects the smoothing parameter by bgacv; bgacv = 0 uses the 
%user specified smoothing parameter. Default is 1.
%
%output = 1 shows detailed iterative information, output = 0 doesn't show
%the iterative information. Default is 0.
%
%Avoid the following variables names as they are used as global in the
%code: fBasis Y_global lambda_global Fvector
global fBasis;
global Y_global;

if nargin < 2
    fprintf('not enough inputs\n')
    return
end

if nargin < 3
    order_inter = 2;
end

Y_global = Y;
fBasis = gBasisq(X,order_inter);      %generate fBasis
fBasis = sparse(fBasis);
sum_fBasis = sum(fBasis,1);
nonzeroinx_fBasis = find(sum_fBasis ~= 0);
fBasis = fBasis(:,sum_fBasis~= 0);
nbeta = size(fBasis,2);
nmain = size(X,2);
if order_inter == 2
    nmain = length(find(sum_fBasis(:,1:nmain)~=0));
elseif order_inter == 3
    nmaintmp = length(find(sum_fBasis(:,1:nmain)~=0));
    ninter2 = nchoosek(nmain,2);
    ninter2 = length(find(sum_fBasis(:,1:(nmain+ninter2))~=0))-nmaintmp;
    nmain = [nmaintmp; ninter2];
end

if nargin <= 5
    output = 0;
    if nargin <=4
        bgacv = 1;
        if nargin <= 3
        % on the tuning parameter of interactions
            if nbeta < 64
                loglambda0(2) = -1.8; %-1.8;  %-2.4
            elseif nbeta < 6000;
                loglambda0(2) = -2; %-2;  %-2.1
            elseif nbeta < 10000;
                loglambda0(2) = -1.3;
            elseif nbeta < 100000;
                loglambda0(2) = -1.1;
            else
                loglambda0(2) = -0.9;
            end
        % on the tuning parameter of main effects
            if nmain < 64
                loglambda0(1) = -2;%-2;  %-2.4
            elseif nbeta < 6000;
                loglambda0(1) = -2.1;
            else
                loglambda0(1) = -1.3;
            end
        end
    end
end
tic
beta = betaoptmt(nmain,loglambda0,bgacv,output);
lassologit = fBasis*beta;
ind = find(abs(beta)>1e-6)';
nsigb = length(ind);
if nsigb > 1
    ind = ind(1:nsigb-1);
    XX = fBasis(:,ind);
    clear fBasis
    XX = full(XX);
    [ncoe nstat sigind] = bgacvp(XX,Y_global);
    runtime = toc;
    logit = [ones(length(Y),1) XX(:,sigind)]*ncoe;
    %save the results
    cluster = gcluster(size(X,2),order_inter);
    siginx_final = nonzeroinx_fBasis(ind(sigind));
    cluster = cluster(siginx_final,:);
    cluster1 = [zeros(1,size(cluster,2)); cluster];
    cluster = cluster1; clear cluster1;
    cluster(:,1) = ncoe;
    lasso.nbrunknowns = length(beta);
    lasso.index = nonzeroinx_fBasis(ind);
    lasso.coeff = beta;
    lasso.prob = exp(lassologit)./(1+exp(lassologit));
    fm.lasso = lasso;
    model.coeff = ncoe;
    model.se = nstat.se;
    model.coeffcorr = nstat.coeffcorr;
    model.index = siginx_final;
    model.p = nstat.p;
    model.prob = exp(logit)./(1+exp(logit));
    fm.model = model;
    fm.pattern = cluster(:,2:size(cluster,2));
    info.sizex = size(X,2);
    info.order = order_inter;
    info.runtime = runtime;
    info.haspattern = 1;
    fm.info = info;
else
    info.haspattern = 0;
    fm.info = info;
end

return

